#!/usr/bin/python
# -*- coding: utf-8 -*-
"""File used to unit test the pacifica_cart."""
import os
import logging
from tempfile import mkdtemp
import cherrypy
from cherrypy.test import helper
import requests
from pacifica.cartd.orm import Cart
from pacifica.cartd.rest import CartRoot, error_page_default
from pacifica.cartd.tasks import CART_APP
from pacifica.cartd.globals import CHERRYPY_CONFIG
from cart_db_setup_test import cart_dbsetup_gen

CART_APP.conf.CELERY_ALWAYS_EAGER = True


# there's a lot of testing with this class suckit pylint
# pylint: disable=too-many-public-methods
class TestRest(cart_dbsetup_gen(helper.CPWebCase)):
    """Contain all the tests for the Cart Interface."""

    PORT = 8081
    HOST = '127.0.0.1'
    url = 'http://{0}:{1}'.format(HOST, PORT)
    headers = {'content-type': 'application/json'}

    # pylint: disable=invalid-name
    @classmethod
    def tearDownClass(cls):
        """Unset the VOLUME_PATH for future tests."""
        del os.environ['VOLUME_PATH']
    # pylint: enable=invalid-name

    @staticmethod
    def setup_server():
        """Start all the services."""
        logger = logging.getLogger('urllib2')
        logger.setLevel(logging.DEBUG)
        logger.addHandler(logging.StreamHandler())
        os.environ['VOLUME_PATH'] = '{}{}'.format(mkdtemp(), os.path.sep)
        cherrypy.config.update({'error_page.default': error_page_default})
        cherrypy.config.update(CHERRYPY_CONFIG)
        cherrypy.tree.mount(CartRoot(), '/', CHERRYPY_CONFIG)

    def test_cart_int_get(self):
        """Testing the cart interface get method w/o file_wrapper."""
        sample_cart = Cart()
        sample_cart.cart_uid = 123
        sample_cart.bundle_path = mkdtemp('', os.environ['VOLUME_PATH'])
        sample_cart.status = 'ready'
        sample_cart.save(force_insert=True)
        req = requests.get('{}/123'.format(self.url))
        self.assertEqual(req.status_code, 200)

    def test_invalid_cart_uid(self):
        """Testing the cart interface get against not valid cart uid."""
        req = requests.get('{}/123'.format(self.url))
        self.assertEqual(req.status_code, 404)

    def test_not_valid_cart(self):
        """Testing the cart interface get against not ready cart."""
        sample_cart = Cart()
        sample_cart.cart_uid = 123
        sample_cart.status = 'ready'
        sample_cart.save(force_insert=True)
        req = requests.get('{}/123'.format(self.url))
        self.assertEqual(req.status_code, 404)

    def test_not_ready_cart(self):
        """Testing the cart interface get against not ready cart."""
        sample_cart = Cart()
        sample_cart.cart_uid = 123
        sample_cart.bundle_path = mkdtemp('', os.environ['VOLUME_PATH'])
        sample_cart.save(force_insert=True)
        req = requests.head('{}/123'.format(self.url))
        self.assertEqual(req.status_code, 204)
        req = requests.get('{}/123'.format(self.url))
        self.assertEqual(req.status_code, 202)
        self.assertEqual(
            req.text,
            'The cart is not ready for download.',
            'the right text came out.'
        )

    def test_status_invalid_uid(self):
        """Testing the cart interface status."""
        req = requests.head('{}/'.format(self.url))
        self.assertEqual(req.status_code, 500)

    def test_cart_int_delete(self):
        """Testing the cart interface delete."""
        sample_cart = Cart()
        sample_cart.cart_uid = 123
        sample_cart.bundle_path = mkdtemp('', os.environ['VOLUME_PATH'])
        sample_cart.status = 'ready'
        sample_cart.save(force_insert=True)
        sample_cart.reload()
        req = requests.delete('{}/123'.format(self.url))
        self.assertEqual(req.status_code, 200)

    def test_delete_invalid_uid(self):
        """Testing the cart interface delete with invalid uid."""
        req = requests.delete('{}/123'.format(self.url))
        self.assertEqual(req.status_code, 404)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Module that is used by the cart to send requests to the archive interface."""
from __future__ import absolute_import
from json import dumps
import hashlib
import requests
from .config import get_config


class ArchiveRequests(object):
    """Class that supports all the requests to the archive interface."""

    def __init__(self):
        """Constructor for setting the AI URL."""
        self._url = get_config().get('archiveinterface', 'url')

    def pull_file(self, archive_filename, cart_filepath, hashval, hashtype):
        """
        Pull file from AI.

        Performs a request that will attempt to write
        the contents of a file from the archive interface
        to the specified cart filepath
        """
        resp = requests.get(str(self._url + archive_filename), stream=True)
        myfile = open(cart_filepath, 'wb+')
        buf = resp.raw.read(1024)
        myhash = hashlib.new(hashtype)
        while buf:
            myfile.write(buf)
            myhash.update(buf)
            buf = resp.raw.read(1024)
        myfile.close()
        myhashval = myhash.hexdigest()
        if myhashval != hashval:
            raise ValueError('File hash does not match provided hash')

    def stage_file(self, file_name):
        """Send a post to the archive interface telling it to stage the file."""
        resp = requests.post(str(self._url + file_name))
        if str(resp.status_code) == '500':
            raise requests.exceptions.RequestException(str(dumps(resp.text)))

    @staticmethod
    def _status_dict(headers, file_name):
        """Return status dictionary from http response headers."""
        return {
            'message': headers['x-pacifica-messsage'],
            'file': file_name,
            'filesize': headers['x-content-length'],
            'ctime': headers['x-pacifica-ctime'],
            'mtime': headers['last-modified'],
            'bytes_per_level': headers['x-pacifica-bytes-per-level'],
            'file_storage_media': headers['x-pacifica-file-storage-media']
        }

    def status_file(self, file_name):
        """Get a status from the  archive interface via Head and returns response."""
        resp = requests.head(str(self._url + file_name))
        return dumps(self._status_dict(resp.headers, file_name))



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""File that will tests the requests and coverage of the server and the tasks."""
from __future__ import print_function
import unittest
import sys
import os
import time
import tarfile
import json
from urllib3.util.retry import Retry
import requests
from requests.adapters import HTTPAdapter
from pacifica.cartd.tasks import CART_APP
from pacifica.cartd.utils import Cartutils
from pacifica.cartd.rest import bytes_type
from pacifica.cartd.tasks import pull_file


def cart_json_helper():
    """Helper that returns a cart json text string."""
    return json.dumps({
        'fileids': [
            {
                'id': 'foo.txt',
                'path': '1/2/3/foo.txt',
                'hashtype': 'md5',
                'hashsum': 'ac59bb32dac432674dd6e620a6b35ff3'
            },
            {
                'id': 'bar.csv',
                'path': '1/2/3/bar.csv',
                'hashtype': 'md5',
                'hashsum': 'ef39aa7f8df8bdc8b8d4d81f4e0ef566'
            },
            {
                'id': 'baz.ini',
                'path': '2/3/4/baz.ini',
                'hashtype': 'md5',
                'hashsum': 'b0c21625a5ef364864191e5907d7afb4'
            }
        ]
    })


class TestCartEndToEnd(unittest.TestCase):
    """Contains all the tests for the end to end cart testing."""

    def setUp(self):
        """Make the tasks not asynchronise for testing."""
        CART_APP.conf.update(CELERY_ALWAYS_EAGER=True)
        session = requests.Session()
        retries = Retry(total=5, backoff_factor=5.0)
        session.mount('http://', HTTPAdapter(max_retries=retries))
        self.session = session

    def setup_good_cart(self, cart_id):
        """Setup a test good cart."""
        resp = self.session.post(
            'http://127.0.0.1:8081/{}'.format(cart_id),
            data=cart_json_helper(),
            headers={
                'Content-Type': 'application/json'
            }
        )
        self.assertEqual(resp.status_code, 201,
                         'Setup good cart for test failed.')
        return resp

    def test_post_cart(self, cart_id='36'):
        """Test the creation of a cart."""
        resp = self.setup_good_cart(cart_id)
        self.assertEqual(resp.json()['message'], 'Cart Processing has begun')

    def test_post_cart_bundle(self, cart_id='36a'):
        """Test the creation of a cart."""
        cart_data = json.loads(cart_json_helper())
        cart_data['bundle'] = True
        resp = self.session.post(
            'http://127.0.0.1:8081/{}'.format(cart_id),
            data=json.dumps(cart_data),
            headers={
                'Content-Type': 'application/json'
            }
        )
        self.assertEqual(
            resp.status_code, 201,
            'Setup good cart for test failed.'
        )
        self.assertEqual(resp.json()['message'], 'Cart Processing has begun')
        time.sleep(5)
        resp = self.session.get(
            'http://127.0.0.1:8081/{}?filename={}'.format(cart_id, cart_id))
        with open(cart_id, 'wb') as fdesc:
            for chunk in resp.iter_content(chunk_size=128):
                fdesc.write(chunk)
        self.assertEqual(os.path.isfile(cart_id), True)
        saved_tar = tarfile.open(cart_id)
        tar_members = saved_tar.getnames()
        self.assertTrue('36a/1/2/3/foo.txt' in tar_members,
                        '{} should have foo.txt in it'.format(tar_members))

    def test_status_cart(self, cart_id='37'):
        """Test the status of a cart."""
        self.test_post_cart(cart_id)

        while True:
            resp = self.session.head(
                'http://127.0.0.1:8081/{}'.format(cart_id))
            resp_status = resp.headers['X-Pacifica-Status']
            resp_message = resp.headers['X-Pacifica-Message']
            resp_code = resp.status_code
            if resp_code == 204 and resp_status != 'staging':
                break
            if resp_code == 500:  # pragma: no cover
                print(resp_message, file=sys.stderr)
                break
            time.sleep(2)

        self.assertEqual(resp_status, 'ready')
        self.assertEqual(resp_message, '')

    def test_get_cart(self, cart_id='38'):
        """Test the getting of a cart."""
        self.test_status_cart(cart_id)

        resp = self.session.get(
            'http://127.0.0.1:8081/{}?filename={}'.format(cart_id, cart_id))
        with open(cart_id, 'wb') as fdesc:
            for chunk in resp.iter_content(chunk_size=128):
                fdesc.write(chunk)

        self.assertEqual(os.path.isfile(cart_id), True)
        saved_tar = tarfile.open(cart_id)
        tar_members = saved_tar.getnames()
        self.assertTrue('38/1/2/3/foo.txt' in tar_members,
                        '{} should have foo.txt in it'.format(tar_members))
        data = saved_tar.extractfile(
            saved_tar.getmember('38/1/2/3/foo.txt')).read()
        self.assertEqual(data, bytes_type('Writing content for first file'))

    def test_get_noncart(self, cart_id='86'):
        """Test the getting of a cart."""
        resp = self.session.get('http://127.0.0.1:8081/{}'.format(cart_id))
        self.assertEqual(
            resp.json()['message'], 'The cart does not exist or has already been deleted')
        self.assertEqual(resp.status_code, 404)

    def test_delete_cart(self, cart_id='39'):
        """Test the deletion of a cart."""
        self.test_status_cart(cart_id)

        resp = self.session.delete('http://127.0.0.1:8081/{}'.format(cart_id))
        self.assertEqual(resp.json()['message'], 'Cart Deleted Successfully')

    def test_delete_invalid_cart(self, cart_id='393'):
        """Test the deletion of a invalid cart."""
        resp = self.session.delete('http://127.0.0.1:8081/{}'.format(cart_id))
        self.assertEqual(resp.json()['message'], 'Not Found')

    def test_pull_invalid_file(self):
        """Test pulling a file id that doesnt exist."""
        pull_file('8765', 'some/bad/path', '1111', False)
        # no action happens on invalid file, so no assertion to check
        self.assertEqual(True, True)

    def test_tar_invalid_cart(self):
        """Test pulling a file id that doesnt exist."""
        cart_utils = Cartutils()
        cart_utils.tar_files('8765')
        # no action happens on invalid cart to tar, so no assertion to check
        self.assertEqual(True, True)

    def test_status_cart_notfound(self):
        """Test the status of a cart with cart not found."""
        cart_id = '97'
        resp = self.session.head('http://127.0.0.1:8081/{}'.format(cart_id))
        resp_status = resp.headers['X-Pacifica-Status']
        resp_message = resp.headers['X-Pacifica-Message']
        resp_code = resp.status_code

        self.assertEqual(resp_status, 'error')
        self.assertEqual(resp_message, 'No cart with uid 97 found')
        self.assertEqual(resp_code, 404)

    def test_status_cart_error(self):
        """Test the status of a cart with error."""
        cart_id = '98'
        bad_cart_data = {
            'fileids': [
                {
                    'id': 'mytest.txt',
                    'path': '1/2/3/mytest.txt',
                    'hashtype': 'md5',
                    'hashsum': ''
                }
            ]
        }

        resp = self.session.post(
            'http://127.0.0.1:8081/{}'.format(cart_id),
            data=json.dumps(bad_cart_data),
            headers={
                'Content-Type': 'application/json'
            }
        )
        self.assertEqual(resp.json()['message'], 'Cart Processing has begun')

        while True:  # pragma: no cover
            resp = self.session.head(
                'http://127.0.0.1:8081/{}'.format(cart_id))
            resp_status = resp.headers['X-Pacifica-Status']
            resp_code = resp.status_code
            if (resp_code == 204 and resp_status != 'staging') or resp_code == 500:
                break
            time.sleep(2)

        self.assertEqual(resp_status, 'error')
        self.assertEqual(resp_code, 500)

    def test_post_cart_bad_hash(self, cart_id='1136'):
        """Test the creation of a cart with bad hash."""
        bad_cart_data = {
            'fileids': [
                {
                    'id': 'foo.txt',
                    'path': '1/2/3/foo.txt',
                    'hashtype': 'md5',
                    'hashsum': 'ac59bb32'
                },
                {
                    'id': 'bar.csv',
                    'path': '1/2/3/bar.csv',
                    'hashtype': 'md5',
                    'hashsum': 'ef39aa7f8df8bdc8b8d4d81f4e0ef566'
                },
                {
                    'id': 'baz.ini',
                    'path': '2/3/4/baz.ini',
                    'hashtype': 'md5',
                    'hashsum': 'b0c21625a5ef364864191e5907d7afb4'
                }
            ]
        }
        resp = self.session.post(
            'http://127.0.0.1:8081/{}'.format(cart_id),
            data=json.dumps(bad_cart_data),
            headers={
                'Content-Type': 'application/json'
            }
        )
        self.assertEqual(resp.json()['message'], 'Cart Processing has begun')

        while True:
            resp = self.session.head(
                'http://127.0.0.1:8081/{}'.format(cart_id))
            resp_status = resp.headers['X-Pacifica-Status']
            resp_code = resp.status_code
            if resp_code == 204 and resp_status != 'staging':  # pragma: no cover
                break
            if resp_code == 500:
                break
            time.sleep(2)

        self.assertEqual(resp_status, 'error')



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""File used to unit test the pacifica_cart."""
import unittest
import os
import sys


class TestEnvGlobals(unittest.TestCase):
    """Contain the tests for the global config module."""

    def test_set_logging(self):
        """Test that logging gets set for debugging."""
        os.environ['DATABASE_LOGGING'] = 'True'
        import importlib
        # first delete the module from the loaded modules
        del sys.modules['pacifica.cartd.globals']
        # then we import the module
        mod = importlib.import_module('pacifica.cartd.globals')
        # make sure the LOGGER attribute in the module exists
        self.assertTrue(getattr(mod, 'CHERRYPY_CONFIG'))



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test cart database setup class."""
from peewee import SqliteDatabase
from pacifica.cartd.orm import Cart, File


def cart_dbsetup_gen(parent_cls):
    """Generate a class with parent and call super on setup for parent."""
    class CartDBSetup(parent_cls):
        """Contain all the tests for the Cart Interface."""

        @staticmethod
        def create_sample_cart(cart_uid='1', status='staging', bundle_path='/tmp/1/'):
            """Create a sample cart."""
            return Cart.create(
                cart_uid=cart_uid,
                status=status,
                bundle_path=bundle_path
            )

        @staticmethod
        def create_sample_file(test_cart, file_name='1.txt', bundle_path='/tmp/1/1.txt'):
            """Create a sample file in cart."""
            return File.create(
                cart=test_cart,
                file_name=file_name,
                bundle_path=bundle_path
            )

        # pylint: disable=invalid-name
        def setUp(self):
            """Setup the database with in memory sqlite."""
            self._db = SqliteDatabase('file:cachedb?mode=memory&cache=shared')
            for model in [Cart, File]:
                model.bind(self._db, bind_refs=False, bind_backrefs=False)
            self._db.connect()
            self._db.create_tables([Cart, File])

        def tearDown(self):
            """Tear down the database."""
            self._db.drop_tables([Cart, File])
            self._db.close()
            self._db = None
        # pylint: enable=invalid-name
    return CartDBSetup



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Add Unit Tests for archive interface."""
import unittest
from json import dumps, loads
from tempfile import mkdtemp
import httpretty
import requests
from pacifica.cartd.archive_requests import ArchiveRequests


class TestArchiveRequests(unittest.TestCase):
    """Test the archive requests class."""

    endpoint_url = 'http://127.0.0.1:8080'

    @httpretty.activate
    def test_archive_get(self):
        """Test pulling a file."""
        response_body = 'This is the body of the file in the archive.'
        httpretty.register_uri(httpretty.GET, '{}/1'.format(self.endpoint_url),
                               body=response_body,
                               content_type='application/octet-stream')
        temp_dir = mkdtemp()
        archreq = ArchiveRequests()
        hashval = '5bf018b3c598df19b5f4363fc55f2f89'
        hashtype = 'md5'
        archreq.pull_file('1', '{}/1'.format(temp_dir), hashval, hashtype)
        testfd = open('{}/1'.format(temp_dir), 'rb')
        self.assertEqual(testfd.read().decode('UTF-8'), response_body)

    @httpretty.activate
    def test_archive_stage(self):
        """Test staging a file."""
        response_body = {
            'message': 'File was staged',
            'file': '1'
        }
        httpretty.register_uri(httpretty.POST, '{}/1'.format(self.endpoint_url),
                               body=dumps(response_body),
                               content_type='application/json')
        archreq = ArchiveRequests()
        archreq.stage_file('1')
        self.assertEqual(httpretty.last_request().method, 'POST')

    @httpretty.activate
    def test_archive_status(self):
        """Test pulling a file."""
        stage_file_obj = {
            'bytes_per_level': '0',
            'ctime': 'Sun, 06 Nov 1994 08:49:37 GMT',
            'file_storage_media': '0',
            'filesize': '8',
            'file': '1',
            'mtime': 'Sun, 06 Nov 1994 08:49:37 GMT',
            'message': 'File was found'
        }
        adding_headers = {
            'x-pacifica-messsage': 'File was found',
            'x-pacifica-ctime': 'Sun, 06 Nov 1994 08:49:37 GMT',
            'x-pacifica-bytes-per-level': '0',
            'x-pacifica-file-storage-media': '0',
            'x-content-length': '8',
            'last-modified': 'Sun, 06 Nov 1994 08:49:37 GMT'
        }
        httpretty.register_uri(httpretty.HEAD, '{}/1'.format(self.endpoint_url),
                               status=200,
                               body='blahblah',
                               adding_headers=adding_headers)
        archreq = ArchiveRequests()
        status = loads(archreq.status_file('1'))
        for key in status.keys():
            self.assertEqual(status[key], stage_file_obj[key])
        self.assertEqual(httpretty.last_request().method, 'HEAD')

    @httpretty.activate
    def test_archive_stage_fail(self):
        """Test staging a file failure."""
        response_body = {
            'message': 'error',
            'file': '1'
        }
        httpretty.register_uri(httpretty.POST, '{}/1'.format(self.endpoint_url),
                               status=500,
                               body=dumps(response_body),
                               content_type='application/json')
        archreq = ArchiveRequests()
        with self.assertRaises(requests.exceptions.RequestException):
            archreq.stage_file('fakeFileName')

    @httpretty.activate
    def test_archive_get_bad_hash(self):
        """Test pulling a file with a bad hash value."""
        response_body = 'This is the body of the file in the archive.'
        httpretty.register_uri(httpretty.GET, '{}/1'.format(self.endpoint_url),
                               body=response_body,
                               content_type='application/octet-stream')
        temp_dir = mkdtemp()
        archreq = ArchiveRequests()
        hashval = '5b'
        hashtype = 'md5'
        with self.assertRaises(ValueError):
            archreq.pull_file('1', '{}/1'.format(temp_dir), hashval, hashtype)
